var webserver;
var entries = [];
var storage_dir;

window.onload = function () {

    //For convencience
    webserver = opera.io.webserver

    if (webserver)
    {
        //Handle requests for various URLs
        webserver.addEventListener('_index', showEntryList, false);
        webserver.addEventListener('entry', showEntry, false);
        webserver.addEventListener('form', showForm, false);
        webserver.addEventListener('save', saveEntry, false);

        // Make the private storage directory available to us
        storage_dir = opera.io.filesystem.mountSystemDirectory('storage');

        var entries_dir = storage_dir.resolve('entries');
        if(!entries_dir.exists)
            storage_dir.createDirectory('entries');

        // Load Entries
        entries_dir.refresh();
        for(var i=0; i<entries_dir.length; i++)
        {
            var entry = entries_dir[i];
            var id = parseInt(entries_dir[i].name);

            var filestream = entries_dir[i].open(null,'r');
            var title = filestream.readLine();
            var date = filestream.readLine();
            var text = filestream.read(filestream.bytesAvailable);
            filestream.close();

            entries[id] = {
                'title' : title,
                'text'  : text,
                'date'  : date
            };
        }
    }
}

/**
* Show the list of currently saved entries.
* This is displayed on the root of the service.
*/
function showEntryList(e)
{
    var response = e.connection.response;
    response.write( '<!DOCTYPE html>'
        + '<html><head><title>Entries</title></head>'
        + '<body><ul>'
    );

    for ( var i = 0, entry; entry = entries[i]; i++ )
    {
        response.write('<li>'+entry.date+': <a href="entry?id='+i+'">'+entry.title+'</a></li>');
    }

    response.write('</ul>'
    + '<p><a href="form">Add an entry</a>.</p>'
    + '</body></html>'
    );
    response.close();
}

/**
* Show a single entry, identified by an id GET parameter.
*/
function showEntry(e)
{
    var index = e.connection.request.queryItems['id'][0];
    var entry = entries[index];
    //XXX Should have error handling here
    var response = e.connection.response;
    response.write('<!DOCTYPE html>'
        + '<html><head><title>'+entry.title+'</title></head>'
        + '<body><h1>'+entry.title+'</h1>'
        + '<p>'+entry.date+'</p>'
        + '<div>'+entry.text+'</div>'
        + '</body></html>'
    );
    response.close();
}

/**
* Show a form for adding an entry.
* When submitted, this will request the 'save' URL.
*/
function showForm(e)
{
    var response = e.connection.response;
    response.write('<!DOCTYPE html>'
        + '<html><head><title>Add entry</title></head>'
        + '<body><h1>Add entry</h1>'
        + '<form method="post" action="save">'
        + '<p><label for="namefield">Title</label> <input id="nameField" type="text" name="title"></p>'
        + '<p><label for="textArea">Text</label> <textarea id="textArea" name="text"></textarea></p>'
        + '<p><input type="submit" name="Add entry"></p>'
        + '</form>'
        + '</body></html>'
    );
    response.close();
}

/**
* Save an entry with data from POST parameters.
* Redirects back to the index when done.
*/
function saveEntry(e)
{
    var request = e.connection.request;
    var response = e.connection.response;

    //Get POST data
    //XXX Should have error handling here
    var title = request.bodyItems['title'][0];
    var text = request.bodyItems['text'][0];

    entries.push({
        'title' : title, 
        'text'  : text,
        'date'  : new Date()
    });

    // Write this entry to a file
    var id = entries.length-1;
    var filestream = storage_dir.open('entries/' + id, 'w');
    filestream.writeLine(entries[id].title);
    filestream.writeLine(entries[id].date);
    filestream.write(entries[id].text);
    filestream.close();

    //Redirect back to the index of the service
    //TODO Pass some success or error message
    response.setStatusCode(302);
    response.setResponseHeader( 'Location', webserver.currentServicePath );
    response.close();
}
